@@ -1,14 +1,15 @@
 ;;; develock.el --- additional font-lock keywords for the developers
 
-;; Copyright (C) 2001, 2002, 2003, 2005 Katsumi Yamaoka
+;; Copyright (C) 2001, 2002, 2003, 2005, 2006 Katsumi Yamaoka
 
 ;; Author: Katsumi Yamaoka  <yamaoka@jpl.org>
 ;;         Jun'ichi Shiono  <jun@fsas.fujitsu.com>
 ;;         Yasutaka SHINDOH <ring-pub@fan.gr.jp>
+;;         Oscar Bonilla    <ob@bitmover.com>
 ;; Created: 2001/06/28
-;; Revised: 2006/01/23
+;; Revised: 2006/10/02
 ;; Keywords: font-lock emacs-lisp change-log texinfo c java perl html
-;;           mail news
+;;           tcl ruby mail news
 
 ;; Develock is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -31,8 +32,9 @@
 ;; lock highlight leading and trailing whitespace, long lines and
 ;; oddities in the file buffer for Lisp modes, ChangeLog mode, Texinfo
 ;; mode, C modes, Java mode, Jde-mode , CPerl mode, Perl mode, HTML
-;; modes and some Mail modes.  Here is an example of how to set up
-;; your startup file (possibly .emacs) to use Develock:
+;; modes, some Mail modes, Tcl mode and Ruby mode.  Here is an example
+;; of how to set up your startup file (possibly .emacs) to use
+;; Develock:
 ;;
 ;;(cond ((featurep 'xemacs)
 ;;       (require 'develock)
@@ -66,7 +68,9 @@
 ;;       (add-hook 'java-mode-hook 'turn-on-font-lock)
 ;;       (add-hook 'html-mode-hook 'turn-on-font-lock)
 ;;       (add-hook 'html-helper-mode-hook 'turn-on-font-lock)
-;;       (add-hook 'message-mode-hook 'turn-on-font-lock)))
+;;       (add-hook 'message-mode-hook 'turn-on-font-lock)
+;;       (add-hook 'tcl-mode-hook 'turn-on-font-lock)
+;;       (add-hook 'ruby-mode-hook 'turn-on-font-lock)))
 ;;
 ;; Note that `jde-mode' activates the `font-lock-mode' by default
 ;; because of the `jde-use-font-lock' variable.
@@ -209,7 +213,7 @@
 
 ;;; Code:
 
-(defconst develock-version "0.27"
+(defconst develock-version "0.30"
   "Version number for this version of Develock.")
 
 (require 'advice)
@@ -401,7 +405,9 @@ That would be defenseless to spammers."
 	'perl-mode 79
 	'mail-mode t
 	'message-mode t
-	'cmail-mail-mode t)
+	'cmail-mail-mode t
+	'tcl-mode 79
+	'ruby-mode 79)
   "Plist of `major-mode's and limitation values for long lines.
 The part of a line that is longer than the limitation value according
 to the `major-mode' is highlighted.  Value `w' means one subtracted
@@ -426,16 +432,33 @@ the value nil."
   "Alist of `major-mode's and the values for `fill-column'.
 When Develock is turned on, `auto-fill-mode' is automatically enabled
 in the buffer where `major-mode' included in this list runs.
-Each value is applied to `fill-column' in the buffer."
+Each value is applied to `fill-column' in the buffer.  If `fill-column'
+is specified as a local variable and is allowed in the buffer, this
+variable will be made buffer-local and the value will be modified."
   :type '(repeat (cons :format "%v"
 		       (symbol :tag "Major mode")
 		       (integer :tag "Fill column")))
   :group 'develock
   :group 'font-lock)
 
+(defadvice hack-local-variables (around develock-allow-local-variables
+					activate)
+  "Advised by Develock.
+Merge local veriables into `develock-fill-column-alist'."
+  (let ((fc fill-column))
+    ad-do-it
+    (if (and (assq major-mode develock-fill-column-alist)
+	     (not (eq fc fill-column)))
+	(progn
+	  (set (make-local-variable 'develock-fill-column-alist)
+	       (copy-sequence develock-fill-column-alist))
+	  (let ((elem (memq (assq major-mode develock-fill-column-alist)
+			    develock-fill-column-alist)))
+	    (setcar elem (cons major-mode fill-column)))))))
+
 (defcustom develock-mode-ignore-kinsoku-list
   '(emacs-lisp-mode lisp-interaction-mode c-mode c++-mode java-mode jde-mode
-		    cperl-mode perl-mode)
+		    cperl-mode perl-mode tcl-mode)
   "List of `major-mode's that ignore kinsoku at the end of lines."
   :type '(repeat (symbol :format "Major-Mode: %v\n" :size 0))
   :set 'develock-custom-set-and-refontify
@@ -553,6 +576,12 @@ try the following advice in your startup file.
 (defvar html-font-lock-keywords-x nil
   "Extraordinary level font-lock keywords for the HTML mode.")
 
+(defvar tcl-font-lock-keywords-x nil
+  "Extraordinary level font-lock keywords for the Tcl mode.")
+
+(defvar ruby-font-lock-keywords-x nil
+  "Extraordinary level font-lock keywords for the Ruby mode.")
+
 (defvar develock-keywords-alist
   '((emacs-lisp-mode lisp-font-lock-keywords-x
 		     develock-lisp-font-lock-keywords)
@@ -583,7 +612,11 @@ try the following advice in your startup file.
     (message-mode message-font-lock-keywords-x
 		  develock-mail-font-lock-keywords)
     (cmail-mail-mode cmail-font-lock-keywords-x
-		     develock-mail-font-lock-keywords))
+		     develock-mail-font-lock-keywords)
+    (tcl-mode tcl-font-lock-keywords-x
+	      develock-tcl-font-lock-keywords)
+    (ruby-mode ruby-font-lock-keywords-x
+	       develock-ruby-font-lock-keywords))
   "*Alist of keyword symbols for major modes.
 Each element should be triple symbols of the following form:
 
@@ -1062,6 +1095,68 @@ Each element should be triple symbols of the following form:
   :group 'develock
   :group 'font-lock)
 
+(defcustom develock-tcl-font-lock-keywords
+  '(;; a long line
+    (develock-find-long-lines
+     (1 'develock-long-line-face-1 t)
+     (2 'develock-long-line-face-2 t))
+    ;; long spaces
+    (develock-find-tab-or-long-space
+     (1 'develock-whitespace-face-2)
+     (2 'develock-whitespace-face-3 nil t))
+    ;; trailing whitespace
+    ("[^\t\n ]\\([\t ]+\\)$"
+     (1 'develock-whitespace-face-1 t))
+    ;; spaces before tabs
+    ("\\( +\\)\\(\t+\\)"
+     (1 'develock-whitespace-face-1 t)
+     (2 'develock-whitespace-face-2 t))
+    ;; tab space tab
+    ("\\(\t\\) \t"
+     (1 'develock-whitespace-face-2 append))
+    ;; only tabs or spaces in the line
+    ("^[\t ]+$"
+     (0 'develock-whitespace-face-2 append))
+    ;; reachable E-mail addresses
+    ("<?[-+.0-9A-Z_a-z]+@[-0-9A-Z_a-z]+\\(\\.[-0-9A-Z_a-z]+\\)+>?"
+     (0 'develock-reachable-mail-address-face t)))
+  "Extraordinary level highlighting for the Tcl mode."
+  :type develock-keywords-custom-type
+  :set 'develock-keywords-custom-set
+  :group 'develock
+  :group 'font-lock)
+
+(defcustom develock-ruby-font-lock-keywords
+  '(;; a long line
+    (develock-find-long-lines
+     (1 'develock-long-line-face-1 t)
+     (2 'develock-long-line-face-2 t))
+    ;; long spaces
+    (develock-find-tab-or-long-space
+     (1 'develock-whitespace-face-2)
+     (2 'develock-whitespace-face-3 nil t))
+    ;; trailing whitespace
+    ("[^\t\n ]\\([\t ]+\\)$"
+     (1 'develock-whitespace-face-1 t))
+    ;; spaces before tabs
+    ("\\( +\\)\\(\t+\\)"
+     (1 'develock-whitespace-face-1 t)
+     (2 'develock-whitespace-face-2 t))
+    ;; tab space tab
+    ("\\(\t\\) \t"
+     (1 'develock-whitespace-face-2 append))
+    ;; only tabs or spaces in the line
+    ("^[\t ]+$"
+     (0 'develock-whitespace-face-2 append))
+    ;; reachable E-mail addresses
+    ("<?[-+.0-9A-Z_a-z]+@[-0-9A-Z_a-z]+\\(\\.[-0-9A-Z_a-z]+\\)+>?"
+     (0 'develock-reachable-mail-address-face t)))
+  "Extraordinary level highlighting for the Ruby mode."
+  :type develock-keywords-custom-type
+  :set 'develock-keywords-custom-set
+  :group 'develock
+  :group 'font-lock)
+
 (defmacro develock-find-font-lock-defaults (modesym)
   "Get the font-lock defaults based on the major mode."
   (if (featurep 'xemacs)
@@ -1576,7 +1671,9 @@ IGNORE-POINT-POS are ignored."
 	  (looking-at "[\t ]*")
 	  (or (setq mod (buffer-modified-p))
 	      (setq orig (match-string 0)))
-	  (delete-region (point) (match-end 0))
+	  (if indent-tabs-mode
+	      (tabify (point) (match-end 0))
+	    (untabify (point) (match-end 0)))
 	  (develock-Orig-c-indent-line syntax quiet ignore-point-pos)
 	  (setq pt (point))
 	  (or mod
